#!/usr/bin/env bash
DRY_RUN=false
SUCCESS_PROJECTS=()
PRETAGGED_PROJECTS=()
ERROR_PROJECTS=()

usage() {
    cat <<EOF
Usage: xivo-multi-tag <version.subversion> [--dry-run] <packager1> <packager2> ...
 
 - <version.subversion> mandatory, numeric version : 2017.03.02 for instance
 
 - --dry-run optionnal, if provided will show you the steps without actually tagging.
 
 - <packager1> <packager2> mandatory : the list of packager projects to tag with that version
 
 - -h, --help, will display this help if passed as first argument.
EOF
}

# shellcheck disable=SC2155
cmn_echo_info() {
  local green=$(tput setaf 2)
  local reset=$(tput sgr0)
  # shellcheck disable=SC2145
  echo -e "${green}$@${reset}"
}

# shellcheck disable=SC2155
cmn_echo_important() {
  local yellow=$(tput setaf 3)
  local reset=$(tput sgr0)
  # shellcheck disable=SC2145
  echo -e "${yellow}$@${reset}"
}

# shellcheck disable=SC2155
cmn_echo_warn() {
  local red=$(tput setaf 1)
  local reset=$(tput sgr0)
# shellcheck disable=SC2145
  echo -e "${red}$@${reset}"
}

cmn_assert_command_is_available() {
  local cmd=${1}
  # shellcheck disable=SC2086
  if ! type ${cmd} >/dev/null 2>&1; then
    echo ""
    cmn_echo_warn "Cancelling because required command '${cmd}' is not available."
    exit 1
  fi
}

verify_help_asked() {
    if [ "$1" == "-h" ] || [ "$1" == "--help" ] ; then
        usage
        exit 0 
    fi
}

verify_version() {
    if [[ ! ${1} =~ [0-9]{4}\.[0-9]{2} ]]; then
        echo "Error: Version ${1} is not a valid xivo.solutions format"
        exit 1
    fi
}

verify_numeric_version() {
    if [[ ! ${1} =~ [0-9]{4}\.[0-9]{2}\.[0-9]{2} ]]; then
        echo "Error: Version.Subversion ${1} is not a valid xivo.solutions format"
        exit 1
    fi
}

set_branch_to_tag() {
    version_table=(2022.10 2022.05 2021.15 2021.07 2020.18 2020.07 2019.12 2019.05 2018.16 2018.05 2017.11 2017.03)

    if [[ "${version_table[*]}" =~ ${1} ]]; then
        branch_to_tag="${1}"
    else
        branch_to_tag="master"
    fi
}

echo_dry_run() {
    cmn_echo_info "DRY RUN : $1"
}

verify_dry_run() {
    if [ "$1" == "--dry-run" ]; then
        DRY_RUN=true
        echo_dry_run "dry run mode on"
    else
        cmn_echo_warn "dry run mode off, be careful"
    fi
}

main() {
    verify_help_asked "$1"
    version="$1"; shift
    verify_version "${version}"
    major_version=$(echo "${version}" | grep -o '^.\{0,7\}')
    set_branch_to_tag "${major_version}"

    if [[ "${version}" = "${major_version}" ]];then
        numeric_version="${major_version}.00"
    else
        verify_numeric_version "${version}"
        numeric_version=$(echo "${version}" | grep -o '^.\{0,10\}')
    fi
    cmn_echo_info "Each packager project will be tagged on branch $branch_to_tag with version ${numeric_version}"
    cmn_assert_command_is_available "xivo-tag-package"

    packager_project="$1"; shift
    verify_dry_run "$packager_project"
    if [ "$DRY_RUN" = true ];then
        packager_project="$1"; shift
    fi
    while [[ -n ${packager_project} ]]
    do 
        echo " "
        cmn_echo_important "You are about to tag "
        cmn_echo_info "project ${packager_project}"
        cmn_echo_important "on branch ${branch_to_tag}
     with type rc
     with version ${numeric_version}"
        if [[ ! -d "${packager_project}" ]];then
            git clone "git@gitlab.com:xivo.solutions/${packager_project}.git" || ERROR_PROJECTS+=("$packager_project") 
        fi
        if [[ -d "${packager_project}" ]]; then
            # shellcheck disable=SC2164
            cd "${packager_project}"
            git pull
            if [ "$DRY_RUN" = true ];then
                xivo-tag-package "${numeric_version}" "rc" "${branch_to_tag}" --dry-run
                TAG_STATUS=$?
            else
                xivo-tag-package "${numeric_version}" "rc" "${branch_to_tag}"
                TAG_STATUS=$?
            fi
            if [ "$TAG_STATUS" == "0" ]; then
                SUCCESS_PROJECTS+=("$packager_project")
            else
                if [ "$TAG_STATUS" == "42" ];then
                    PRETAGGED_PROJECTS+=("$packager_project")
                else
                    ERROR_PROJECTS+=("$packager_project")
                fi
            fi
            # shellcheck disable=SC2103
            cd ..
        fi
        packager_project="$1"; shift
        echo " "
    done
    cmn_echo_important "Congratulations ! Multi-tagging done."
    cmn_echo_info "These projects have been tagged successfully: ${SUCCESS_PROJECTS[*]}"
    cmn_echo_important "These projects were already tagged: ${PRETAGGED_PROJECTS[*]}"
    cmn_echo_warn "These projects have encountered issues while tagging: ${ERROR_PROJECTS[*]}"
}

main "$@"